package renderer.primitive;

import basics.math.algebra.Ray;
import basics.math.algebra.Vector;

public class Sphere implements Primitive {

    private Vector center;
    private double radius;

    public Sphere(Vector center, double radius) {
        this.center = center;
        this.radius = radius;
    }

    @Override
    public double intersect(Ray ray) {
        Vector oc = ray.getOrigin().sub(this.center);

        double b = ray.getDirection().dot(oc);
        double c = oc.dot(oc) - this.radius * this.radius;
        double h = b * b - c;

        if (h < 0.0)
            return -1.0;

        return -b - h;
    }

    @Override
    public Vector normalAt(Vector surfacePoint) {
        return surfacePoint.sub(center).normalize();
    }
}
